
import sys

try:
    import pkg_resources
    #
    # Load modules associated with Plugins that are defined in
    # EGG files.
    #
    for entrypoint in pkg_resources.iter_entry_points('coopr.command'):
        plugin_class = entrypoint.load()
except Exception:
    err = sys.exc_info()[1]
    sys.stderr.write( "Error loading coopr.command entry points: %s  entrypoint='%s'\n" % (err, entrypoint) )


import coopr_parser


def main(args=None):
    parser = coopr_parser.get_parser()

    if args is None:
        ret = parser.parse_args()
    else:
        ret = parser.parse_args(args)
    ret.func(ret)





__all__ = ['add_subparser', 'get_parser']

import argparse
import warnings
import sys

#
# Sort sub_parser names, since these are inserted throughout Coopr
#
# NOTE: This may not be robust to different versions of argparse.  We're
# mucking with a non-public API here ...
#
class CustomHelpFormatter(argparse.RawDescriptionHelpFormatter):

    def _metavar_formatter(self, action, default_metavar):
        if action.metavar is not None:
            result = action.metavar
        elif action.choices is not None:
            choice_strs = sorted(str(choice) for choice in action.choices)
            result = '{%s}' % ','.join(choice_strs)
        else:
            result = default_metavar

        def format(tuple_size):
            if isinstance(result, tuple):
                return result
            else:
                return (result, ) * tuple_size
        return format

    def _iter_indented_subactions(self, action):
        try:
            get_subactions = action._get_subactions
        except AttributeError:
            pass
        else:
            self._indent()
            if isinstance(action, argparse._SubParsersAction):
                for subaction in sorted(get_subactions(), key=lambda x: x.dest):
                    yield subaction
            else:
                for subaction in get_subactions():
                    yield subaction
            self._dedent()

#
# `BaseException.message` is deprecated as of Python 2.6, its usage triggers
# a `DeprecationWarning`.  As `ArgumentError` derives indirectly from
# `BaseException`, `ArgumentError.message` triggers this warning too
#
if sys.version_info[:2] == (2,6):
    warnings.filterwarnings(
        'ignore',
        message='BaseException.message has been deprecated as of Python 2.6',
        category=DeprecationWarning,
        module='argparse')


doc="This is the top-level command for the Coopr optimization project."
epilog="""
-------------------------------------------------------------------------
Coopr supports a variety of modeling and optimization capabilities.
Different tools are executed as subcommands of this command.  Each
subcommand supports independent command-line options.  Use the -h option
to print details for a subcommand.  For example, type

   coopr pyomo -h

to print information about the `pyomo` subcommand.
"""
coopr_parser = argparse.ArgumentParser(description=doc, epilog=epilog, formatter_class=CustomHelpFormatter)
coopr_subparsers = coopr_parser.add_subparsers(dest='subparser_name', title='subcommands')

def add_subparser(name, **args):
    """Add a subparser to the 'coopr' command."""
    func = None
    if 'func' in args:
        func = args['func']
        del args['func']
    parser = coopr_subparsers.add_parser(name, **args)
    if not func is None:
        parser.set_defaults(func=func)
    return parser

def get_parser():
    """Return the parser used by the 'coopr' commmand."""
    return coopr_parser





import argparse
import coopr_parser
import os
import os.path
import sys
import glob
import datetime
import textwrap
import pyutilib.subprocess

def setup_command_parser(parser):
    parser.add_argument("--list", dest="summary", action='store_true', default=False,
                        help="List the commands that are installed with Coopr")
    parser.add_argument("command", nargs='*', help="The command and command-line options")

def command_exec(options):
    cmddir = os.path.dirname(os.path.abspath(sys.executable))+os.sep
    if options.summary:
        print("")
        print("The following commands are installed in the Coopr bin directory:")
        print("----------------------------------------------------------------")
        for file in sorted(glob.glob(cmddir+'*')):
            print(" "+os.path.basename(file))
        print("")
        if len(options.command) > 0:
            print("WARNING: ignoring command specification")
        return
    if len(options.command) == 0:
        print("ERROR: no command specified")
        return
    if not os.path.exists(cmddir+options.command[0]):
        print("ERROR: the command '%s' does not exist" % (cmddir+options.command[0]))
        return
    pyutilib.subprocess.run(cmddir+' '.join(options.command), tee=True)

def version_exec(options):
    from coopr.coopr import version
    import platform
    print("Coopr %s (%s %s on %s %s)" % (
            version,
            platform.python_implementation(),
            '.'.join( str(x) for x in sys.version_info[:3] ), 
            platform.system(),
            platform.release() ) )

def setup_api_parser(parser):
    parser.add_argument("-a", "--asciidoc", dest="asciidoc", action='store_true', default=False,
                        help="Output the API summary using ASCIIDOC syntax")

def api_exec(options):
    import coopr.core
    services = coopr.core.CooprAPIFactory.services()
    #
    f = {}
    for name in services:
        f[name] = coopr.core.CooprAPIFactory(name)
    #
    ns = {}
    for name in services:
        ns_set = ns.setdefault(f[name].__namespace__, set())
        ns_set.add(name)
    #
    if options.asciidoc:
        print("//")
        print("// Coopr Library API Documentation")
        print("//")
        print("// Generated with 'coopr api' on ",datetime.date.today())
        print("//")
        print("")
        print("== Coopr Functor API ==")
        for ns_ in sorted(ns.keys()):
            print("")
            level = ns_+" Functors"
            print('=== %s ===' % level)
            for name in sorted(ns[ns_]):
                if ns_ != '':
                    tname = name[len(ns_)+1:]
                else:
                    tname = name
                print("")
                print('==== %s ====' % tname)
                print(f[name].__short_doc__)
                if f[name].__long_doc__ != '':
                    print("")
                    print(f[name].__long_doc__)
                print("")
                flag=False
                print("- [underline]#Required Keyword Arguments:#")
                for port in sorted(f[name].inputs):
                    if f[name].inputs[port].optional:
                        flag=True
                        continue
                    print("")
                    print('*%s*::\n %s' % (port, f[name].inputs[port].doc))
                if flag:
                    # A function may not have optional arguments
                    print("")
                    print("- [underline]#Optional Keyword Arguments:#")
                    for port in sorted(f[name].inputs):
                        if not f[name].inputs[port].optional:
                            continue
                        print("")
                        print('*%s*::\n %s' % (port, f[name].inputs[port].doc))
                print("")
                print("- [underline]#Return Values:#")
                for port in sorted(f[name].outputs):
                    print("")
                    print('*%s*::\n %s' % (port, f[name].outputs[port].doc))
                print("")
    else:
        print("")
        print("Coopr Functor API")
        print("-----------------")
        wrapper = textwrap.TextWrapper(subsequent_indent='')
        print(wrapper.fill("The Coopr library contains a set of functors that define operations that are likely to be major steps in Coopr scripts.  This API is defined with functors to ensure a consistent function syntax.  Additionally, these functors can be accessed with a factory, thereby avoiding the need to import modules throughout Coopr."))
        print("")
        for ns_ in sorted(ns.keys()):
            print("")
            level = ns_+" Functors"
            print("-"*len(level))
            print(level)
            print("-"*len(level))
            for name in sorted(ns[ns_]):
                if ns_ != '':
                    tname = name[len(ns_)+1:]
                else:
                    tname = name
                print(tname+':')
                for line in f[name].__short_doc__.split('\n'):
                    print("    "+line)

#
# Add a subparser for the coopr command
#
setup_api_parser(
coopr_parser.add_subparser('api',
        func=api_exec, 
        help='Print information about the Coopr Library API.',
        description='This coopr subcommand is used to print a summary of the Coopr Library API.'))

setup_command_parser(
    coopr_parser.add_subparser('run',
        func=command_exec, 
        help='Execute a command from the Coopr bin (or Scripts) directory.',
        description='This coopr subcommand is used to execute commands installed with Coopr.',
        epilog="""
This subcommand can execute any command from the bin (or Script)
directory that is created when Coopr is installed.  Note that this
includes any commands that are installed by other Python packages
that are installed with Coopr.  Thus, if Coopr is installed in the
Python system directories, then this command executes any command
included with Python.
"""
        ))

coopr_parser.add_subparser('version',
        func=version_exec, 
        help='Print the Coopr version.',
        description='This coopr subcommand is used to print the version of the Coopr release.')

def main(args=None):
    parser = argparse.ArgumentParser()
    setup_parser(parser)
    parser.set_defaults(func=main_exec)
    ret = parser.parse_args(args)
    ret = ret.func(ret)

